"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createCompileBabelStep = createCompileBabelStep;

var _path = require("path");

var _ui = require("@sewing-kit/ui");

var _pluginUtilities = require("@sewing-kit/plugin-utilities");

function createCompileBabelStep(pkg, workspace, config, options) {
  return (0, _ui.createStep)(async step => {
    const {
      configFile = 'babel.js',
      outputPath
    } = options;
    const babelConfigPath = workspace.internal.configPath(`build/packages/${pkg.name}/${configFile}`);

    if (config.babelConfig == null || config.babelIgnorePatterns == null) {
      throw new _pluginUtilities.MissingPluginError('@sewing-kit/plugin-babel');
    }

    const [babelConfig, babelIgnorePatterns] = await Promise.all([config.babelConfig.promise({}), config.babelIgnorePatterns.promise([])]);
    await workspace.internal.write(babelConfigPath, `module.exports=${JSON.stringify(babelConfig)};`);
    const extensions = await config.extensions.promise([]);
    const sourceRoot = (0, _path.resolve)(pkg.root, 'src');
    await step.exec('node_modules/.bin/babel', [sourceRoot, ...(0, _pluginUtilities.toArgs)({
      outDir: outputPath,
      configFile: babelConfigPath,
      verbose: true,
      noBabelrc: true,
      babelConfig: false,
      extensions: extensions.join(','),
      ignore: babelIgnorePatterns.join(',')
    }, {
      dasherize: true
    })]);
  });
}