"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const ui_1 = require("@sewing-kit/ui");
const plugin_utilities_1 = require("@sewing-kit/plugin-utilities");
function createCompileBabelStep(pkg, workspace, config, options) {
    return ui_1.createStep(async (step) => {
        const { configFile = 'babel.js', outputPath } = options;
        const babelConfigPath = workspace.internal.configPath(`build/packages/${pkg.name}/${configFile}`);
        if (config.babelConfig == null) {
            throw new plugin_utilities_1.MissingPluginError('@sewing-kit/plugin-babel');
        }
        const babelConfig = await config.babelConfig.promise({ presets: [] });
        await workspace.internal.write(babelConfigPath, `module.exports=${JSON.stringify(babelConfig)};`);
        const extensions = await config.extensions.promise([]);
        const sourceRoot = path_1.resolve(pkg.root, 'src');
        await step.exec('node_modules/.bin/babel', [
            sourceRoot,
            ...plugin_utilities_1.toArgs({
                outDir: outputPath,
                configFile: babelConfigPath,
                verbose: true,
                noBabelrc: true,
                babelConfig: false,
                extensions: extensions.join(','),
            }, { dasherize: true }),
        ]);
    });
}
exports.createCompileBabelStep = createCompileBabelStep;
